//-----------------------------------------------------------------------------------------------------------
//--PIR activated CameraTraps using cheap 6$ 808 keychain camera (simalar to #22-27) and arduino as controler


#include <avr/sleep.h>      //native avr library
#include <avr/power.h>      //native avr library
#include <avr/wdt.h>        //native avr library

#define powerON 10          // arduino pin10
#define acionButton 11      // arduino pin11
#define ModeSelectButton 9  // arduino pin9

#define PIRsens 3           // arduino pin3; int1
#define LEDindicator 13     // arduino buld in led on pin13


//***settings***
#define fotsk 3                   // multishoot count
#define powerONdelay 6000         // power on time to ready to operate
#define FotoButtonPressTime 100   // acionButton Button Press Time in miliseconds to take foto
#define fotoTakeTime 1000         // delay time to save foto or separatinion time in multishoot (use at least 1000ms to save foto)
#define WaitPIR 3000              // PIR ready time

boolean ModeSelectButtonState = 0;
boolean ModeSelectButtonLastState = 0;

byte mode = 0;                           // Cam select mode //0-PIR picture; 1-PIR video; 2-time lapse



void setup()
{
	// arduino pinout setup
	pinMode(powerON, OUTPUT);
	digitalWrite(powerON, LOW);
	pinMode(acionButton, OUTPUT);
	digitalWrite(acionButton, LOW);

	pinMode(LEDindicator, OUTPUT);
	digitalWrite(LEDindicator, LOW);

	pinMode(ModeSelectButton, INPUT);
	pinMode(PIRsens, INPUT_PULLUP);

	// serial output setup
	Serial.begin(9600); 
	Serial.println("start");

	blinkLED(3, 500);   // blink Led 3 times
	delay(WaitPIR);     // PIR ready time
	// modeSetup();
}



ISR(WDT_vect) {
	//sleep timer wakeup with wachdog timer
}



void wakeUpNow() {
	//sleep Now, wakeup with external pin cange interupt
}


void sleepNow() {

	sleep_enable();                          // enables the sleep bit in the mcucr register  
	attachInterrupt(1, wakeUpNow, FALLING);  // use interrupt 1 (pin 3) and run function  
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);     // setting sleep mode 									     
	sleep_mode();                            // go to sleep
	/* ...sleeping... */
	sleep_disable();                         // disable sleep
	detachInterrupt(1);                      // disables interrupt 1 on pin 2 so the wakeUpNow code will not be executed during normal running time.  
}

void sleepTimer() {
	WDTCSR |= (1 << WDCE) | (1 << WDE);    // set WDCE register
	WDTCSR |= (1 << WDIE);                 // Enable the Wach Dog interrupt
	sleep_enable();                        // enables the sleep bit in the mcucr register  
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);   // setting sleep mode 
	sleep_mode();                          // go to sleep
	/* ...sleepipg... */
	sleep_disable();                       // disable sleep
}
void blinkLED(byte n, int time) {
	for (byte x = 0; x < n; x++) {
		digitalWrite(LEDindicator, HIGH);
		Serial.print(x + 1);
		delay(time);
		digitalWrite(LEDindicator, LOW);
		delay(time);
	}
	Serial.println(" ");
	digitalWrite(LEDindicator, LOW);
}

void loop() {

	Serial.println("");
	Serial.println("loop start");

	blinkLED(3, 100);

	if (mode == 0) {
		takeFoto();          // mode foto PIR

	}
	else if (mode == 1) {
		takeVideo();         // mode video PIR
	}
	else if (mode == 2) {
		timeLapse();         // timelapse
	}
	else {
		Serial.println("err mode");
		delay(500);
	}
}


void takeFoto() {
	digitalWrite(powerON, HIGH);               // turn on camera
	delay(powerONdelay);                       // wait for camera to be ready
	do {
		for (byte x = 0; x < fotsk; x++) {
			digitalWrite(acionButton, HIGH);   // pressing camera action button
			delay(FotoButtonPressTime);        // wait for button press to be registred
			digitalWrite(acionButton, LOW);    // realease button
			blinkLED(2, 100);        
			Serial.println(" foto");
			delay(fotoTakeTime);               // wait to save foto to memory card
		}
	} while (digitalRead(PIRsens) == LOW);     // loop tke foto sequince while PIR is active
	digitalWrite(powerON, LOW);                // if PIR not active turn off camera
	delay(200);

	Serial.println("PIR not active");
	Serial.println("going sleep");

	delay(100);
	sleepNow();     // go to sleep
}

void takeVideo() {

	int m = 0;
	digitalWrite(acionButton, HIGH);
	delay(2000);
	digitalWrite(acionButton, LOW);
	Serial.println("videoOn");

	wdt_enable(WDTO_8S);  // wake up every 8 seconnd to check PIR sensor

	do {
		Serial.println("videoSleep");
		delay(100);
		sleepTimer();
		Serial.println("video wakeup ");
		m++;
		Serial.print(" video");
		Serial.println(m);
	} while (digitalRead(PIRsens) == LOW);

	wdt_disable();
	digitalWrite(acionButton, HIGH); //stop recording video
	delay(200);
	digitalWrite(acionButton, LOW);
	delay(2000);
	digitalWrite(powerON, LOW); // camera power off

	Serial.println("videoStop");
	m = 0;

	Serial.println("PIR not active");
	Serial.println("going sleep");
	delay(100);
	sleepNow();     // call sleep function  
}

void timeLapse() {
	
	digitalWrite(powerON, HIGH);       // turn on camera
	delay(powerONdelay);               // wait for camera to be ready

	digitalWrite(acionButton, HIGH);   // pressing camera action button
	delay(FotoButtonPressTime);        // wait for button press to be registred
	digitalWrite(acionButton, LOW);    // realease button

	int n = 0;                         // variable setup
	n++;                               // incrament +1 every loop
	Serial.print(n);                   // serial print foto number
	Serial.println(" T_foto");

	delay(fotoTakeTime);               // wait to save foto to memory card

	digitalWrite(powerON, LOW);        // turn camera off // to be ready, camera must wait "powerONdelay"

	wdt_enable(WDTO_8S);               // spleep for 8s (WDTO_8S); 4s (WDTO_4S); 2s (WDTO_2S);   
	Serial.println("timelapseSleep");
	delay(100);
	sleepTimer();                      // call sleep function
	Serial.println("timelapseWakeup");
	wdt_disable();
}



void modeSetup() {
	int sutupTimeOut = 0;
	blinkLED(5, 100);
	Serial.println("mode select meniu");
	while (sutupTimeOut < 2000) {

		sutupTimeOut++;
		ModeSelectButtonState = digitalRead(ModeSelectButton);
		if (ModeSelectButtonState != ModeSelectButtonLastState) {

			if (ModeSelectButtonState == HIGH) { // if the current state is HIGH then the button pressed  
				mode++;

				if (mode > 2) {
					mode = 0; // roll over mode settings
				}
				sutupTimeOut = 0;
				Serial.print("mode ");
				Serial.println(mode);
			}

			delay(80); // Delay to avoid bouncing
		}
		ModeSelectButtonLastState = ModeSelectButtonState; // save the current state as the last state,


		Serial.println(sutupTimeOut);
		delay(1);
	}
	Serial.print("mode ");
	Serial.println(mode);
	blinkLED(1 + mode, 500);
}
